==> spread-5.3.0.5-py3-none-any.whl: spread_core/bam/dali/tuner.py <==
from spread_core.tools import settings

from spread_core.tools.timer import Timer

from spread_core.protocols.dali.gear import general as gear
from .providers import lights


class TunerLimited:
    UP = None
    DOWN = None

    @classmethod
    def update_config(cls):
        cls.UP = gear.Up if settings.config.get(settings.TUNING_BY_RATES, False) else gear.StepUp
        cls.DOWN = gear.Down if settings.config.get(settings.TUNING_BY_RATES, False) else gear.StepDown

    def __init__(self, owner):
        if self.__class__.UP is None or self.__class__.DOWN is None:
            self.__class__.update_config()

        self._owner = owner
        self._cmd = None
        self.timer = None
        self.speed = 0.1
        self.count = 0
        self._paused = False
        self._delay = 0

    def set_direction(self, step_class, speed):
        self._cmd = step_class(self._owner.short_addr)
        self.speed = speed
        self.count = 0
        self.set_tunned_flag_for_not_device(True)

        self._start_timer(self._owner.fadeTime)

    def check_state(self, cmd):
        if self._cmd is None:
            self._stop_timer()

        elif isinstance(cmd, (self.__class__.UP, self.__class__.DOWN, gear.OnAndStepUp, gear.StepDownAndOff)):
            _c_level = self._owner.get_value(lights.CurrentLevelRaw, 0)
            _c_min = self._owner.get_value(lights.MinLevelRaw, 1)
            _c_max = self._owner.get_value(lights.MaxLevelRaw, 254)

            if self.count not in range(0, 255):
                self.stop()
            elif isinstance(cmd, (self.__class__.UP, gear.OnAndStepUp)) and self._owner.is_device and _c_level+1 not in range(_c_min, _c_max+1):
                self.stop()
            elif isinstance(cmd, (self.__class__.DOWN, gear.StepDownAndOff)) and self._owner.is_device and _c_level-1 not in range(_c_min, _c_max+1):
                self.stop()
            elif not self.timer:
                self._start_timer(self.speed, increment_count=True)

        else:
            self._start_timer(self._owner.fadeTime)

    def step(self):
        self._stop_timer()

        if self._cmd is not None:
            cmd = self._cmd
            if cmd is not None:
                if self._delay > 0:
                    self._start_timer(self._delay)
                    self._delay = 0
                else:
                    self._paused = False
                    self._owner.on_command_prepared(cmd)

    def stop(self):
        self._cmd = None
        self._delay = 0
        self._owner.set_value(lights.Tuning, 0)
        self._stop_timer()
        self.set_tunned_flag_for_not_device(False)

    def _start_timer(self, delay, increment_count=False):
        self._stop_timer()
        self.timer = Timer.set(delay, self.step)
        if increment_count:
            self.count += 1

    def _stop_timer(self):
        if self.timer:
            try:
                self.timer.close()
            except ValueError:
                pass
            finally:
                self.timer = None

    def set_tunned_flag_for_not_device(self, flag):
        for _provider in self._owner.get_members():
            _provider.is_unaddress_tunned = flag

    def set_pause(self, delay: float):
        self._paused = True
        self._delay = delay

    @property
    def is_active(self): return not self._paused and self._cmd is not None

==> spread-5.8.1.80-py3-none-any.whl: spread_core/bam/dali/tuner.py <==
import spread_dali.gear.general as gear
from spread_core.tools import settings
from spread_core.tools.timer import Timer
from .providers import lights


class TunerLimited:
    UP = None
    DOWN = None

    @classmethod
    def update_config(cls):
        if settings.config.get(settings.TUNING_BY_RATES, False):
            cls.UP = gear.Up
            cls.DOWN = gear.Down
        else:
            cls.UP = gear.StepUp
            cls.DOWN = gear.StepDown

    def __init__(self, owner):
        if self.__class__.UP is None or self.__class__.DOWN is None:
            self.__class__.update_config()

        self._owner = owner
        self._cmd = None
        self.timer = None
        self.speed = 0.1
        self.count = 0
        self._paused = False
        self._delay = 0

    def set_direction(self, step_class, speed):
        self._cmd = step_class(self._owner.binding)
        self.speed = speed
        self.count = 0
        self.set_tunned_flag_for_not_device(True)

        self._start_timer(self._owner.fadeTime)

    def check_state(self, cmd):
        if self._cmd is None:
            self._stop_timer()
        else:
            _c_level = self._owner.get_value(lights.CurrentLevelRaw, 0)
            _c_min = self._owner.get_value(lights.MinLevelRaw, 1)
            _c_max = self._owner.get_value(lights.MaxLevelRaw, 254)

            if isinstance(self._cmd, gear.OnAndStepUp) and self._owner.is_device and _c_level >= _c_max:
                self.stop()

            elif isinstance(self._cmd, gear.StepDownAndOff) and self._owner.is_device and _c_level == 0:
                self.stop()

            elif isinstance(self._cmd, (gear.Up, gear.StepUp)) and self._owner.is_device and not 0 < _c_level < _c_max:
                self.stop()

            elif isinstance(self._cmd, (gear.Down, gear.StepDown)) and self._owner.is_device and _c_level <= _c_min:
                self.stop()

            elif type(self._cmd) is type(cmd):
                if not self.timer and self.count < 254:
                    self._start_timer(self.speed, increment_count=True)
            else:
                self._start_timer(self._owner.fadeTime)

    def step(self):
        self._stop_timer()

        if self._cmd is not None:
            cmd = self._cmd
            if cmd is not None:
                if self._delay > 0:
                    self._start_timer(self._delay)
                    self._delay = 0
                else:
                    self._paused = False
                    self._owner.make_step(cmd)

    def stop(self):
        self._cmd = None
        self._delay = 0
        self._owner.set_value(lights.Tuning, 0)
        self._stop_timer()
        self.set_tunned_flag_for_not_device(False)

    def _start_timer(self, delay, increment_count=False):
        self._stop_timer()
        self.timer = Timer.set(delay, self.step)
        if increment_count:
            self.count += 1

    def _stop_timer(self):
        if self.timer:
            try:
                self.timer.close()
            except ValueError:
                pass
            finally:
                self.timer = None

    def set_tunned_flag_for_not_device(self, flag):
        for _provider in self._owner.get_members():
            _provider.is_unaddress_tunned = flag

    def set_pause(self, delay: float):
        self._paused = True
        if delay > self._delay:
            self._delay = delay

    @property
    def is_active(self): return not self._paused and self._cmd is not None
